from_component

Function from_component 

pub fn from_component<C>(
    entity: impl Into<Entity> + Send + Sync + 'static,
) -> impl Signal<Item = C> + Clone
where C: Component + Clone,
Creates a Signal from an Entity or LazyEntity and a Component, terminating for the frame if the Entity does not exist or the Component does not exist on the Entity.