MapSignal

Struct MapSignal 

pub struct MapSignal<Upstream, S: Signal> { /* private fields */ }
Signal graph node which applies a System to each Item of its upstream, forwarding the output of each resulting Signal, see .map_signal.